define(function(require, exports, module) {
    function XXTea(privateKey) {
        this._keyString = privateKey;
    }

    //将长整形转换为string,private
    XXTea.prototype._long2byte = function(v) {
        var vl = v.length;
        var b = [];
        var j = 0;
        for (var i = 0; i < vl; i++) {
            b[j++] = v[i] & 0xff;
            b[j++] = v[i] >>> 8 & 0xff;
            b[j++] = v[i] >>> 16 & 0xff;
            b[j++] = v[i] >>> 24 & 0xff;
        }

        return b;

    }


    //将长整形转换为string,private
    XXTea.prototype._byte2hexstr = function(b) {

        var bl = b.length;
        var h = '';
        var temp;
        for (var i = 0; i < bl; i++) {
            temp = parseInt(b[i].toString(2), 2).toString(16);
            if (temp.length <= 1) {
                h += '0' + temp;
            } else {
                h += temp;
            }

        }

        return h;

    }

    //将string转换为long,private
    XXTea.prototype._str2long = function(s, byteLength) {
        var len = s.length;
        var v = [];
        for (var i = 0; i < len; i += 4) {
            v[i >> 2] = s.charCodeAt(i) | s.charCodeAt(i + 1) << 8 | s.charCodeAt(i + 2) << 16 | s.charCodeAt(i + 3) << 24;
        }

        if (byteLength) {
            while (v.length < byteLength) {
                v[v.length] = 0;
            }
        }


        return v;
    }


    //function: encrypt str with private key by XXTea
    XXTea.prototype.bteaEncrypt = function(v, k) {
        var n = v.length - 1;

        var z = v[n],
            y = v[0],
            delta = 0x9E3779B9;
        var mx, e, p, q = Math.floor(6 + 52 / (n + 1)),
            sum = 0;
        while (0 < q--) {
            sum = sum + delta & 0xffffffff;
            e = sum >>> 2 & 3;
            for (p = 0; p < n; p++) {
                y = v[p + 1];
                mx = (z >>> 5 ^ y << 2) + (y >>> 3 ^ z << 4) ^ (sum ^ y) + (k[p & 3 ^ e] ^ z);
                z = v[p] = v[p] + mx & 0xffffffff;
            }
            y = v[0];
            mx = (z >>> 5 ^ y << 2) + (y >>> 3 ^ z << 4) ^ (sum ^ y) + (k[p & 3 ^ e] ^ z);
            z = v[n] = v[n] + mx & 0xffffffff;
        }

    }

    //function: encrypt str with private key by XXTea
    XXTea.prototype.encrypt = function(str) {
        if (str == "") {
            return "";
        }

        var XXTEA_ALIGNMENT_BYTES = 8;

        var nLength = str.length;
        nResult = Math.floor(nLength / XXTEA_ALIGNMENT_BYTES) + (nLength % XXTEA_ALIGNMENT_BYTES > 0 ? 1 : 0);

        var byteLength = nResult * 2;

        var v = this._str2long(str, byteLength);
        var k = this._str2long(this._keyString);
        this.bteaEncrypt(v, k);

        var b = this._long2byte(v);

        var h = this._byte2hexstr(b);

        return h;
    }


    //function: decrypt str with private key by XXTea
    XXTea.prototype.bteaDecrypt = function(v, k) {

        var n = v.length - 1;

        var z = v[n - 1],
            y = v[0],
            delta = 0x9E3779B9;
        var mx, e, p, q = Math.floor(6 + 52 / (n + 1)),
            sum = q * delta & 0xffffffff;
        while (sum != 0) {
            e = sum >>> 2 & 3;
            for (p = n; p > 0; p--) {
                z = v[p - 1];
                mx = (z >>> 5 ^ y << 2) + (y >>> 3 ^ z << 4) ^ (sum ^ y) + (k[p & 3 ^ e] ^ z);
                y = v[p] = v[p] - mx & 0xffffffff;
            }
            z = v[n];
            mx = (z >>> 5 ^ y << 2) + (y >>> 3 ^ z << 4) ^ (sum ^ y) + (k[p & 3 ^ e] ^ z);
            y = v[0] = v[0] - mx & 0xffffffff;
            sum = sum - delta & 0xffffffff;
        }

    }

    //function: decrypt str with private key by XXTea
    XXTea.prototype.decrypt = function(str) {
        if (str == "") {
            return "";
        }
        var v = this._str2long(str);
        var k = this._str2long(this._keyString);
        this.btea(v, k);

        return this._long2str(v);
    }

    return XXTea;
})
