define(function(require, exports, module) {
    var getData = require('../../../base/js/getData');
    require('../../../base/js/util/touch/touch');
    require('../css/m_selector.css');

    var Selector = function() {
        var tpl = '<div class="selector-hue brightness g-dispbox">' +
            '    <div class="title">亮度</div>' +
            '    <div class="line-box g-box-flex">' +
            '        <div class="l"></div>' +
            '        <span class="m" id="brightness"></span>' +
            '    </div>' +
            '</div>' +
            '<div class="selector-hue color g-dispbox">' +
            '    <div class="title">色温</div>' +
            '    <div class="line-box g-box-flex">' +
            '        <div class="l"></div>' +
            '        <span class="m" id="color"></span>' +
            '    </div>' +
            '</div>';
        var $selector = $('#selector'); //容器
        var $brightness; //亮度选择器
        var $color; //色温选择器
        var width; //选择的宽度
        var colorData; //颜色数组
        var oldBVal = '0'; //记录原有亮度，默认0
        var oldCVal = 'ff0000'; //记录原有颜色，默认ff0000
        var cmd;
        $selector.html(tpl);

        //init
        function init(bVal, cVal) {
            var bVal = bVal || oldBVal;
            var cVal = cVal || oldCVal;
            bVal = rangeControl(bVal, 255);
            cVal = cVal.toLowerCase();
            colorData = setColorData();
            width = $selector.find('.l').width();
            $brightness = $('#brightness');
            $color = $('#color');
            moveHue(bVal, 'brightness');
            moveHue(cVal, 'color');
            getBrightness($brightness[0]);
            getColor($color[0]);
            oldBVal = bVal;
            oldCVal = cVal;
            return this;
        }

        //是否开启
        function getSwitch(type) {
            var cls = type ? 'on' : 'off';
            $selector.toggleClass(cls);
            return this;
        }

        //对比
        function rangeControl(num, max) {
            num = Math.max(num, 0);
            return Math.min(num, max);
        }

        //亮度选择
        function getBrightness(o) {
            var startX;
            toucher(o).on('swipeStart', function(e) {
                startX = parseInt(this.style.left) || 0;
            }).on('swipe', function(e) {
                this.style.left = rangeControl(startX + e.moveX, width) + 'px';
                return false;
            }).on('swipeEnd', function(e) {
                var bStr;
                var num = (Math.ceil(rangeControl(startX + (e.moveX || 0), width) / (width) * 100));
                var bright = Math.ceil(num * 2.55);
                if (bright !== oldBVal) {
                    bStr = '{"state":"bright","value":"' + bright + '"}';

                    //上报
                    cmd = '{"modId":"' + modId + '","stateSet":[' + bStr + ']}';
                    cmd = JSON.parse(cmd); //

                    getData.getExecuteCmd(cmd, function(data) {
                        if (data.ret === '0') {
                            //console.log(data);
                        } else {
                            bright = oldBVal;
                            gFuncObj.gErrorTips(data.msg);
                        }

                    });
                };
                return false;
            })
        }


        //色温选择
        function getColor(o) {
            var startX;
            toucher(o).on('swipeStart', function(e) {
                startX = parseInt(this.style.left) || 0;
            }).on('swipe', function(e) {
                this.style.left = rangeControl(startX + e.moveX, width) + 'px';
                return false;
            }).on('swipeEnd', function(e) {
                var cStr;
                var n = Math.ceil(rangeControl(startX + (e.moveX || 0), width) / (width) * 100);
                var i = parseInt(Math.floor(n * 0.99));
                if (i !== oldCVal) {
                    cStr = '{"state":"colorTemp","value":"' + colorData[i] + '"}';
                    //上报
                    cmd = '{"modId":"' + modId + '","stateSet":[' + cStr + ']}';
                    cmd = JSON.parse(cmd); //

                    getData.getExecuteCmd(cmd, function(data) {
                        if (data.ret === '0') {
                            //console.log(data);
                        } else {
                            i = oldCVal;
                            //报错信息
                            gFuncObj.gErrorTips(data.msg);
                        }
                    });

                };
            })
        }

        //设置参数位置
        function moveHue(val, type) {
            var index;
            if (type === 'brightness') {
                $brightness.css('left', parseInt(val * width / 100 / 2.55));
            } else if (type === 'color') {
                index = colorData.indexOf(val) < 0 ? 0 : colorData.indexOf(val);
                $color.css('left', indexColor(index));
            }
        }

        //色温条位置
        function indexColor(num) {
            var left;
            left = Math.ceil(num / 0.99) / 100 * width;
            return left;
        }

        //颜色选取值
        function gradientColor(startColor, endColor, step) {
            startRGB = this.colorRgb(startColor); //转换为rgb数组模式
            startR = startRGB[0];
            startG = startRGB[1];
            startB = startRGB[2];

            endRGB = this.colorRgb(endColor);
            endR = endRGB[0];
            endG = endRGB[1];
            endB = endRGB[2];

            sR = (endR - startR) / step; //总差值
            sG = (endG - startG) / step;
            sB = (endB - startB) / step;

            var colorArr = [];
            for (var i = 0; i < step; i++) {
                //计算每一步的hex值
                //var hex = this.colorHex('rgb(' + parseInt((sR * i + startR)) + ',' + parseInt((sG * i + startG)) + ',' + parseInt((sB * i + startB)) + ')');

                var r = parseInt((sR * i + startR)).toString(16);
                var g = parseInt((sG * i + startG)).toString(16);
                var b = parseInt((sB * i + startB)).toString(16);
                r = r.length < 2 ? 0 + '' + r : r;
                g = g.length < 2 ? 0 + '' + g : g;
                b = b.length < 2 ? 0 + '' + b : b;
                var rgb = r + g + b;
                colorArr.push(rgb);
            }
            return colorArr;
        }

        // 将hex表示方式转换为rgb表示方式(这里返回rgb数组模式)
        gradientColor.prototype.colorRgb = function(sColor) {
            var reg = /^#([0-9a-fA-f]{3}|[0-9a-fA-f]{6})$/;
            var sColor = sColor.toLowerCase();
            if (sColor && reg.test(sColor)) {
                if (sColor.length === 4) {
                    var sColorNew = "#";
                    for (var i = 1; i < 4; i += 1) {
                        sColorNew += sColor.slice(i, i + 1).concat(sColor.slice(i, i + 1));
                    }
                    sColor = sColorNew;
                }
                //处理六位的颜色值
                var sColorChange = [];
                for (var i = 1; i < 7; i += 2) {
                    sColorChange.push(parseInt("0x" + sColor.slice(i, i + 2)));
                }
                return sColorChange;
            } else {
                return sColor;
            }
        };

        // 将rgb表示方式转换为hex表示方式
        gradientColor.prototype.colorHex = function(rgb) {
            var _this = rgb;
            var reg = /^#([0-9a-fA-f]{3}|[0-9a-fA-f]{6})$/;
            if (/^(rgb|RGB)/.test(_this)) {
                var aColor = _this.replace(/(?:(|)|rgb|RGB)*/g, "").split(",");
                var strHex = "#";
                for (var i = 0; i < aColor.length; i++) {
                    var hex = Number(aColor[i]).toString(16);
                    hex = hex < 10 ? 0 + '' + hex : hex; // 保证每个rgb的值为2位
                    if (hex === "0") {
                        hex += hex;
                    }
                    strHex += hex;
                }
                if (strHex.length !== 7) {
                    strHex = _this;
                }
                return strHex;
            } else if (reg.test(_this)) {
                var aNum = _this.replace(/#/, "").split("");
                if (aNum.length === 6) {
                    return _this;
                } else if (aNum.length === 3) {
                    var numHex = "#";
                    for (var i = 0; i < aNum.length; i += 1) {
                        numHex += (aNum[i] + aNum[i]);
                    }
                    return numHex;
                }
            } else {
                return _this;
            }
        }

        //合并数组
        function concatArr() {
            return Array.prototype.concat.apply([], arguments)
        }

        //获取颜色组
        function setColorData() {
            var WhiteArr = ['fff5f5', 'ffffff'];
            var cArr;
            var colorArr1 = new gradientColor('#ff0000', '#ff00ff', 14);
            var colorArr2 = new gradientColor('#ff00ff', '#0000ff', 14);
            var colorArr3 = new gradientColor('#0000ff', '#00ffff', 14);
            var colorArr4 = new gradientColor('#00ffff', '#00ff00', 14);
            var colorArr5 = new gradientColor('#00ff00', '#ffff00', 14);
            var colorArr6 = new gradientColor('#ffff00', '#ff0400', 14);
            var colorArr7 = new gradientColor('#ff0400', '#ffffff', 14);
            cArr = concatArr(colorArr1, colorArr2, colorArr3, colorArr4, colorArr5, colorArr6, colorArr7, WhiteArr);
            return cArr;
        }

        return {
            init: init,
            getSwitch: getSwitch
        };
    };

    return new Selector;
});
